@@ -1,4 +1,4 @@
-/* $OpenBSD: ui_lib.c,v 1.32 2017/01/29 17:49:23 beck Exp $ */
+/* $OpenBSD: ui_lib.c,v 1.44 2020/09/25 11:25:31 tb Exp $ */
 /* Written by Richard Levitte (richard@levitte.org) for the OpenSSL
  * project 2001.
  */
@@ -79,26 +79,22 @@ UI_new_method(const UI_METHOD *method)
 {
 	UI *ret;
 
-	ret = malloc(sizeof(UI));
-	if (ret == NULL) {
+	if ((ret = calloc(1, sizeof(UI))) == NULL) {
 		UIerror(ERR_R_MALLOC_FAILURE);
 		return NULL;
 	}
-	if (method == NULL)
+	if ((ret->meth = method) == NULL)
 		ret->meth = UI_get_default_method();
-	else
-		ret->meth = method;
-
-	ret->strings = NULL;
-	ret->user_data = NULL;
-	ret->flags = 0;
 	CRYPTO_new_ex_data(CRYPTO_EX_INDEX_UI, ret, &ret->ex_data);
+
 	return ret;
 }
 
 static void
 free_string(UI_STRING *uis)
 {
+	if (uis == NULL)
+		return;
 	if (uis->flags & OUT_STRING_FREEABLE) {
 		free((char *) uis->out_string);
 		switch (uis->type) {
@@ -128,8 +124,8 @@ static int
 allocate_string_stack(UI *ui)
 {
 	if (ui->strings == NULL) {
-		ui->strings = sk_UI_STRING_new_null();
-		if (ui->strings == NULL) {
+		if ((ui->strings = sk_UI_STRING_new_null()) == NULL) {
+			UIerror(ERR_R_MALLOC_FAILURE);
 			return -1;
 		}
 	}
@@ -137,94 +133,131 @@ allocate_string_stack(UI *ui)
 }
 
 static UI_STRING *
-general_allocate_prompt(UI *ui, const char *prompt, int prompt_freeable,
+general_allocate_prompt(const char *prompt, int dup_prompt,
     enum UI_string_types type, int input_flags, char *result_buf)
 {
-	UI_STRING *ret = NULL;
+	UI_STRING *uis = NULL;
 
 	if (prompt == NULL) {
 		UIerror(ERR_R_PASSED_NULL_PARAMETER);
-	} else if ((type == UIT_PROMPT || type == UIT_VERIFY ||
-	    type == UIT_BOOLEAN) && result_buf == NULL) {
-		UIerror(UI_R_NO_RESULT_BUFFER);
-	} else if ((ret = malloc(sizeof(UI_STRING)))) {
-		ret->out_string = prompt;
-		ret->flags = prompt_freeable ? OUT_STRING_FREEABLE : 0;
-		ret->input_flags = input_flags;
-		ret->type = type;
-		ret->result_buf = result_buf;
+		goto err;
 	}
-	return ret;
+	if ((type == UIT_PROMPT || type == UIT_VERIFY || type == UIT_BOOLEAN) &&
+	    result_buf == NULL) {
+		UIerror(UI_R_NO_RESULT_BUFFER);
+		goto err;
+	}
+
+	if ((uis = calloc(1, sizeof(UI_STRING))) == NULL) {
+		UIerror(ERR_R_MALLOC_FAILURE);
+		goto err;
+	}
+	uis->out_string = prompt;
+	if (dup_prompt) {
+		if ((uis->out_string = strdup(prompt)) == NULL) {
+			UIerror(ERR_R_MALLOC_FAILURE);
+			goto err;
+		}
+		uis->flags = OUT_STRING_FREEABLE;
+	}
+	uis->input_flags = input_flags;
+	uis->type = type;
+	uis->result_buf = result_buf;
+
+	return uis;
+
+ err:
+	free_string(uis);
+	return NULL;
 }
 
 static int
-general_allocate_string(UI *ui, const char *prompt, int prompt_freeable,
+general_allocate_string(UI *ui, const char *prompt, int dup_prompt,
     enum UI_string_types type, int input_flags, char *result_buf, int minsize,
     int maxsize, const char *test_buf)
 {
-	int ret = -1;
-	UI_STRING *s = general_allocate_prompt(ui, prompt, prompt_freeable,
-	    type, input_flags, result_buf);
+	UI_STRING *s;
+	int ret;
+
+	if ((s = general_allocate_prompt(prompt, dup_prompt, type, input_flags,
+	    result_buf)) == NULL)
+		goto err;
+	s->_.string_data.result_minsize = minsize;
+	s->_.string_data.result_maxsize = maxsize;
+	s->_.string_data.test_buf = test_buf;
+
+	if (allocate_string_stack(ui) < 0)
+		goto err;
+	if ((ret = sk_UI_STRING_push(ui->strings, s)) <= 0)
+		goto err;
 
-	if (s) {
-		if (allocate_string_stack(ui) >= 0) {
-			s->_.string_data.result_minsize = minsize;
-			s->_.string_data.result_maxsize = maxsize;
-			s->_.string_data.test_buf = test_buf;
-			ret = sk_UI_STRING_push(ui->strings, s);
-			/* sk_push() returns 0 on error.  Let's adapt that */
-			if (ret <= 0)
-				ret--;
-		} else
-			free_string(s);
-	}
 	return ret;
+
+ err:
+	free_string(s);
+	return -1;
 }
 
 static int
 general_allocate_boolean(UI *ui, const char *prompt, const char *action_desc,
-    const char *ok_chars, const char *cancel_chars, int prompt_freeable,
+    const char *ok_chars, const char *cancel_chars, int dup_strings,
     enum UI_string_types type, int input_flags, char *result_buf)
 {
-	int ret = -1;
-	UI_STRING *s;
-	const char *p;
+	UI_STRING *s = NULL;
+	int ret;
 
-	if (ok_chars == NULL) {
+	if (ok_chars == NULL || cancel_chars == NULL) {
 		UIerror(ERR_R_PASSED_NULL_PARAMETER);
-	} else if (cancel_chars == NULL) {
-		UIerror(ERR_R_PASSED_NULL_PARAMETER);
-	} else {
-		for (p = ok_chars; *p; p++) {
-			if (strchr(cancel_chars, *p)) {
-				UIerror(UI_R_COMMON_OK_AND_CANCEL_CHARACTERS);
+		goto err;
+	}
+	if (ok_chars[strcspn(ok_chars, cancel_chars)] != '\0') {
+		UIerror(UI_R_COMMON_OK_AND_CANCEL_CHARACTERS);
+		goto err;
+	}
+
+	if ((s = general_allocate_prompt(prompt, dup_strings, type, input_flags,
+	    result_buf)) == NULL)
+		goto err;
+
+	if (dup_strings) {
+		if (action_desc != NULL) {
+			if ((s->_.boolean_data.action_desc =
+			    strdup(action_desc)) == NULL) {
+				UIerror(ERR_R_MALLOC_FAILURE);
+				goto err;
 			}
 		}
-
-		s = general_allocate_prompt(ui, prompt, prompt_freeable,
-		    type, input_flags, result_buf);
-
-		if (s) {
-			if (allocate_string_stack(ui) >= 0) {
-				s->_.boolean_data.action_desc = action_desc;
-				s->_.boolean_data.ok_chars = ok_chars;
-				s->_.boolean_data.cancel_chars = cancel_chars;
-				ret = sk_UI_STRING_push(ui->strings, s);
-				/*
-				 * sk_push() returns 0 on error. Let's adapt
-				 * that
-				 */
-				if (ret <= 0)
-					ret--;
-			} else
-				free_string(s);
+		if ((s->_.boolean_data.ok_chars = strdup(ok_chars)) == NULL) {
+			UIerror(ERR_R_MALLOC_FAILURE);
+			goto err;
 		}
+		if ((s->_.boolean_data.cancel_chars = strdup(cancel_chars)) ==
+		    NULL) {
+			UIerror(ERR_R_MALLOC_FAILURE);
+			goto err;
+		}
+	} else {
+		s->_.boolean_data.action_desc = action_desc;
+		s->_.boolean_data.ok_chars = ok_chars;
+		s->_.boolean_data.cancel_chars = cancel_chars;
 	}
+
+	if (allocate_string_stack(ui) < 0)
+		goto err;
+	if ((ret = sk_UI_STRING_push(ui->strings, s)) <= 0)
+		goto err;
+
 	return ret;
+
+ err:
+	free_string(s);
+	return -1;
 }
 
-/* Returns the index to the place in the stack or -1 for error.  Uses a
-   direct reference to the prompt.  */
+/*
+ * Returns the index to the place in the stack or -1 for error.  Uses a
+ * direct reference to the prompt.
+ */
 int
 UI_add_input_string(UI *ui, const char *prompt, int flags, char *result_buf,
     int minsize, int maxsize)
@@ -233,21 +266,12 @@ UI_add_input_string(UI *ui, const char *prompt, int flags, char *result_buf,
 	    result_buf, minsize, maxsize, NULL);
 }
 
-/* Same as UI_add_input_string(), excepts it takes a copy of the prompt */
+/* Same as UI_add_input_string(), excepts it takes a copy of the prompt. */
 int
 UI_dup_input_string(UI *ui, const char *prompt, int flags, char *result_buf,
     int minsize, int maxsize)
 {
-	char *prompt_copy = NULL;
-
-	if (prompt) {
-		prompt_copy = strdup(prompt);
-		if (prompt_copy == NULL) {
-			UIerror(ERR_R_MALLOC_FAILURE);
-			return 0;
-		}
-	}
-	return general_allocate_string(ui, prompt_copy, 1, UIT_PROMPT, flags,
+	return general_allocate_string(ui, prompt, 1, UIT_PROMPT, flags,
 	    result_buf, minsize, maxsize, NULL);
 }
 
@@ -263,16 +287,7 @@ int
 UI_dup_verify_string(UI *ui, const char *prompt, int flags,
     char *result_buf, int minsize, int maxsize, const char *test_buf)
 {
-	char *prompt_copy = NULL;
-
-	if (prompt) {
-		prompt_copy = strdup(prompt);
-		if (prompt_copy == NULL) {
-			UIerror(ERR_R_MALLOC_FAILURE);
-			return -1;
-		}
-	}
-	return general_allocate_string(ui, prompt_copy, 1, UIT_VERIFY, flags,
+	return general_allocate_string(ui, prompt, 1, UIT_VERIFY, flags,
 	    result_buf, minsize, maxsize, test_buf);
 }
 
@@ -288,49 +303,8 @@ int
 UI_dup_input_boolean(UI *ui, const char *prompt, const char *action_desc,
     const char *ok_chars, const char *cancel_chars, int flags, char *result_buf)
 {
-	char *prompt_copy = NULL;
-	char *action_desc_copy = NULL;
-	char *ok_chars_copy = NULL;
-	char *cancel_chars_copy = NULL;
-
-	if (prompt) {
-		prompt_copy = strdup(prompt);
-		if (prompt_copy == NULL) {
-			UIerror(ERR_R_MALLOC_FAILURE);
-			goto err;
-		}
-	}
-	if (action_desc) {
-		action_desc_copy = strdup(action_desc);
-		if (action_desc_copy == NULL) {
-			UIerror(ERR_R_MALLOC_FAILURE);
-			goto err;
-		}
-	}
-	if (ok_chars) {
-		ok_chars_copy = strdup(ok_chars);
-		if (ok_chars_copy == NULL) {
-			UIerror(ERR_R_MALLOC_FAILURE);
-			goto err;
-		}
-	}
-	if (cancel_chars) {
-		cancel_chars_copy = strdup(cancel_chars);
-		if (cancel_chars_copy == NULL) {
-			UIerror(ERR_R_MALLOC_FAILURE);
-			goto err;
-		}
-	}
-	return general_allocate_boolean(ui, prompt_copy, action_desc_copy,
-	    ok_chars_copy, cancel_chars_copy, 1, UIT_BOOLEAN, flags,
-	    result_buf);
-
-err:
-	free(prompt_copy);
-	free(action_desc_copy);
-	free(ok_chars_copy);
-	free(cancel_chars_copy);
-	return -1;
+	return general_allocate_boolean(ui, prompt, action_desc, ok_chars,
+	    cancel_chars, 1, UIT_BOOLEAN, flags, result_buf);
 }
 
 int
@@ -343,17 +317,8 @@ UI_add_info_string(UI *ui, const char *text)
 int
 UI_dup_info_string(UI *ui, const char *text)
 {
-	char *text_copy = NULL;
-
-	if (text) {
-		text_copy = strdup(text);
-		if (text_copy == NULL) {
-			UIerror(ERR_R_MALLOC_FAILURE);
-			return -1;
-		}
-	}
-	return general_allocate_string(ui, text_copy, 1, UIT_INFO, 0, NULL,
-	    0, 0, NULL);
+	return general_allocate_string(ui, text, 1, UIT_INFO, 0, NULL, 0, 0,
+	    NULL);
 }
 
 int
@@ -366,17 +331,8 @@ UI_add_error_string(UI *ui, const char *text)
 int
 UI_dup_error_string(UI *ui, const char *text)
 {
-	char *text_copy = NULL;
-
-	if (text) {
-		text_copy = strdup(text);
-		if (text_copy == NULL) {
-			UIerror(ERR_R_MALLOC_FAILURE);
-			return -1;
-		}
-	}
-	return general_allocate_string(ui, text_copy, 1, UIT_ERROR, 0, NULL,
-	    0, 0, NULL);
+	return general_allocate_string(ui, text, 1, UIT_ERROR, 0, NULL, 0, 0,
+	    NULL);
 }
 
 char *
@@ -433,8 +389,9 @@ UI_get0_result(UI *ui, int i)
 }
 
 static int
-print_error(const char *str, size_t len, UI *ui)
+print_error(const char *str, size_t len, void *arg)
 {
+	UI *ui = arg;
 	UI_STRING uis;
 
 	memset(&uis, 0, sizeof(uis));
@@ -456,9 +413,7 @@ UI_process(UI *ui)
 		return -1;
 
 	if (ui->flags & UI_FLAG_PRINT_ERRORS)
-		ERR_print_errors_cb(
-		    (int (*)(const char *, size_t, void *)) print_error,
-		    (void *)ui);
+		ERR_print_errors_cb(print_error, ui);
 
 	for (i = 0; i < sk_UI_STRING_num(ui->strings); i++) {
 		if (ui->meth->ui_write_string &&
@@ -500,7 +455,7 @@ UI_process(UI *ui)
 		}
 	}
 
-err:
+ err:
 	if (ui->meth->ui_close_session && !ui->meth->ui_close_session(ui))
 		return -1;
 	return ok;
@@ -582,7 +537,7 @@ UI_set_method(UI *ui, const UI_METHOD *meth)
 
 
 UI_METHOD *
-UI_create_method(char *name)
+UI_create_method(const char *name)
 {
 	UI_METHOD *ui_method = calloc(1, sizeof(UI_METHOD));
 
@@ -592,9 +547,11 @@ UI_create_method(char *name)
 	return ui_method;
 }
 
-/* BIG FSCKING WARNING!!!!  If you use this on a statically allocated method
-   (that is, it hasn't been allocated using UI_create_method(), you deserve
-   anything Murphy can throw at you and more!  You have been warned. */
+/*
+ * BIG FSCKING WARNING!!!!  If you use this on a statically allocated method
+ * (that is, it hasn't been allocated using UI_create_method(), you deserve
+ * anything Murphy can throw at you and more!  You have been warned.
+ */
 void
 UI_destroy_method(UI_METHOD *ui_method)
 {
@@ -609,8 +566,8 @@ UI_method_set_opener(UI_METHOD *method, int (*opener)(UI *ui))
 	if (method) {
 		method->ui_open_session = opener;
 		return 0;
-	} else
-		return -1;
+	}
+	return -1;
 }
 
 int
@@ -619,8 +576,8 @@ UI_method_set_writer(UI_METHOD *method, int (*writer)(UI *ui, UI_STRING *uis))
 	if (method) {
 		method->ui_write_string = writer;
 		return 0;
-	} else
-		return -1;
+	}
+	return -1;
 }
 
 int
@@ -629,8 +586,8 @@ UI_method_set_flusher(UI_METHOD *method, int (*flusher)(UI *ui))
 	if (method) {
 		method->ui_flush = flusher;
 		return 0;
-	} else
-		return -1;
+	}
+	return -1;
 }
 
 int
@@ -639,8 +596,8 @@ UI_method_set_reader(UI_METHOD *method, int (*reader)(UI *ui, UI_STRING *uis))
 	if (method) {
 		method->ui_read_string = reader;
 		return 0;
-	} else
-		return -1;
+	}
+	return -1;
 }
 
 int
@@ -649,8 +606,8 @@ UI_method_set_closer(UI_METHOD *method, int (*closer)(UI *ui))
 	if (method) {
 		method->ui_close_session = closer;
 		return 0;
-	} else
-		return -1;
+	}
+	return -1;
 }
 
 int
@@ -661,63 +618,57 @@ UI_method_set_prompt_constructor(UI_METHOD *method,
 	if (method) {
 		method->ui_construct_prompt = prompt_constructor;
 		return 0;
-	} else
-		return -1;
+	}
+	return -1;
 }
 
 int
-(*UI_method_get_opener(UI_METHOD * method))(UI *)
+(*UI_method_get_opener(const UI_METHOD * method))(UI *)
 {
 	if (method)
 		return method->ui_open_session;
-	else
-		return NULL;
+	return NULL;
 }
 
 int
-(*UI_method_get_writer(UI_METHOD *method))(UI *, UI_STRING *)
+(*UI_method_get_writer(const UI_METHOD *method))(UI *, UI_STRING *)
 {
 	if (method)
 		return method->ui_write_string;
-	else
-		return NULL;
+	return NULL;
 }
 
 int
-(*UI_method_get_flusher(UI_METHOD *method)) (UI *)
+(*UI_method_get_flusher(const UI_METHOD *method)) (UI *)
 {
 	if (method)
 		return method->ui_flush;
-	else
-		return NULL;
+	return NULL;
 }
 
 int
-(*UI_method_get_reader(UI_METHOD *method))(UI *, UI_STRING *)
+(*UI_method_get_reader(const UI_METHOD *method))(UI *, UI_STRING *)
 {
 	if (method)
 		return method->ui_read_string;
-	else
-		return NULL;
+	return NULL;
 }
 
 int
-(*UI_method_get_closer(UI_METHOD *method))(UI *)
+(*UI_method_get_closer(const UI_METHOD *method))(UI *)
 {
 	if (method)
 		return method->ui_close_session;
-	else
-		return NULL;
+	return NULL;
 }
 
 char *
-(*UI_method_get_prompt_constructor(UI_METHOD *method))(UI *, const char *,
+(*UI_method_get_prompt_constructor(const UI_METHOD *method))(UI *, const char *,
     const char *)
 {
 	if (method)
 		return method->ui_construct_prompt;
-	else
-		return NULL;
+	return NULL;
 }
 
 enum UI_string_types
@@ -816,6 +767,7 @@ UI_get_result_maxsize(UI_STRING *uis)
 int
 UI_set_result(UI *ui, UI_STRING *uis, const char *result)
 {
+	const char *p;
 	int l = strlen(result);
 
 	ui->flags &= ~UI_FLAG_REDOABLE;
@@ -851,29 +803,25 @@ UI_set_result(UI *ui, UI_STRING *uis, const char *result)
 		    uis->_.string_data.result_maxsize + 1);
 		break;
 	case UIT_BOOLEAN:
-		{
-			const char *p;
-
-			if (!uis->result_buf) {
-				UIerror(UI_R_NO_RESULT_BUFFER);
-				return -1;
-			}
-			uis->result_buf[0] = '\0';
-			for (p = result; *p; p++) {
-				if (strchr(uis->_.boolean_data.ok_chars, *p)) {
-					uis->result_buf[0] =
-					    uis->_.boolean_data.ok_chars[0];
-					break;
-				}
-				if (strchr(uis->_.boolean_data.cancel_chars, *p)) {
-					uis->result_buf[0] =
-					    uis->_.boolean_data.cancel_chars[0];
-					break;
-				}
-			}
-		default:
-			break;
+		if (!uis->result_buf) {
+			UIerror(UI_R_NO_RESULT_BUFFER);
+			return -1;
 		}
+		uis->result_buf[0] = '\0';
+		for (p = result; *p; p++) {
+			if (strchr(uis->_.boolean_data.ok_chars, *p)) {
+				uis->result_buf[0] =
+				    uis->_.boolean_data.ok_chars[0];
+				break;
+			}
+			if (strchr(uis->_.boolean_data.cancel_chars, *p)) {
+				uis->result_buf[0] =
+				    uis->_.boolean_data.cancel_chars[0];
+				break;
+			}
+		}
+	default:
+		break;
 	}
 	return 0;
 }
