@@ -1,6 +1,6 @@
-/* $OpenBSD: x509_verify.c,v 1.24 2020/11/18 17:54:46 tb Exp $ */
+/* $OpenBSD: x509_verify.c,v 1.36 2021/03/13 23:01:49 tobhe Exp $ */
 /*
- * Copyright (c) 2020 Bob Beck <beck@openbsd.org>
+ * Copyright (c) 2020-2021 Bob Beck <beck@openbsd.org>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -52,7 +52,8 @@ x509_verify_chain_new(void)
 	if ((chain->cert_errors = calloc(X509_VERIFY_MAX_CHAIN_CERTS,
 	    sizeof(int))) == NULL)
 		goto err;
-	if ((chain->names = x509_constraints_names_new()) == NULL)
+	if ((chain->names =
+	    x509_constraints_names_new(X509_VERIFY_MAX_CHAIN_NAMES)) == NULL)
 		goto err;
 
 	return chain;
@@ -195,24 +196,29 @@ x509_verify_ctx_cert_is_root(struct x509_verify_ctx *ctx, X509 *cert)
 
 static int
 x509_verify_ctx_set_xsc_chain(struct x509_verify_ctx *ctx,
-    struct x509_verify_chain *chain, int set_error)
+    struct x509_verify_chain *chain, int set_error, int is_trusted)
 {
-	X509 *last = x509_verify_chain_last(chain);
-	size_t depth;
+	size_t num_untrusted;
 	int i;
 
 	if (ctx->xsc == NULL)
 		return 1;
 
-	depth = sk_X509_num(chain->certs);
-	if (depth > 0)
-		depth--;
+	/*
+	 * XXX last_untrusted is actually the number of untrusted certs at the
+	 * bottom of the chain. This works now since we stop at the first
+	 * trusted cert. This will need fixing once we allow more than one
+	 * trusted certificate.
+	 */
+	num_untrusted = sk_X509_num(chain->certs);
+	if (is_trusted && num_untrusted > 0)
+		num_untrusted--;
+	ctx->xsc->last_untrusted = num_untrusted;
 
-	ctx->xsc->last_untrusted = depth ? depth - 1 : 0;
 	sk_X509_pop_free(ctx->xsc->chain, X509_free);
 	ctx->xsc->chain = X509_chain_up_ref(chain->certs);
 	if (ctx->xsc->chain == NULL)
-		return x509_verify_cert_error(ctx, last, depth,
+		return x509_verify_cert_error(ctx, NULL, 0,
 		    X509_V_ERR_OUT_OF_MEM, 0);
 
 	if (set_error) {
@@ -264,7 +270,7 @@ x509_verify_ctx_add_chain(struct x509_verify_ctx *ctx,
 		ctx->xsc->error = X509_V_OK;
 		ctx->xsc->error_depth = 0;
 
-		if (!x509_verify_ctx_set_xsc_chain(ctx, chain, 0))
+		if (!x509_verify_ctx_set_xsc_chain(ctx, chain, 0, 1))
 			return 0;
 
 		/*
@@ -381,8 +387,18 @@ x509_verify_consider_candidate(struct x509_verify_ctx *ctx, X509 *cert,
 	/* Fail if the certificate is already in the chain */
 	for (i = 0; i < sk_X509_num(current_chain->certs); i++) {
 		if (X509_cmp(sk_X509_value(current_chain->certs, i),
-		    candidate) == 0)
+		    candidate) == 0) {
+			if (is_root_cert) {
+				/*
+				 * Someone made a boo-boo and put their root
+				 * in with their intermediates - handle this
+				 * gracefully as we'll have already picked
+				 * this up as a shorter chain.
+				 */
+				ctx->dump_chain = 1;
+			}
 			return 0;
+		}
 	}
 
 	if (ctx->sig_checks++ > X509_VERIFY_MAX_SIGCHECKS) {
@@ -420,7 +436,7 @@ x509_verify_consider_candidate(struct x509_verify_ctx *ctx, X509 *cert,
 	 * give up.
 	 */
 	if (is_root_cert) {
-		if (!x509_verify_ctx_set_xsc_chain(ctx, new_chain, 0)) {
+		if (!x509_verify_ctx_set_xsc_chain(ctx, new_chain, 0, 1)) {
 			x509_verify_chain_free(new_chain);
 			return 0;
 		}
@@ -460,6 +476,14 @@ x509_verify_build_chains(struct x509_verify_ctx *ctx, X509 *cert,
 	X509 *candidate;
 	int i, depth, count, ret;
 
+	/*
+	 * If we are finding chains with an xsc, just stop after we have
+	 * one chain, there's no point in finding more, it just exercises
+	 * the potentially buggy callback processing in the calling software.
+	 */
+	if (ctx->xsc != NULL && ctx->chains_count > 0)
+		return;
+
 	depth = sk_X509_num(current_chain->certs);
 	if (depth > 0)
 		depth--;
@@ -475,6 +499,7 @@ x509_verify_build_chains(struct x509_verify_ctx *ctx, X509 *cert,
 		return;
 
 	count = ctx->chains_count;
+	ctx->dump_chain = 0;
 	ctx->error = X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT_LOCALLY;
 	ctx->error_depth = depth;
 	if (ctx->xsc != NULL) {
@@ -490,6 +515,7 @@ x509_verify_build_chains(struct x509_verify_ctx *ctx, X509 *cert,
 			    X509_V_ERR_SELF_SIGNED_CERT_IN_CHAIN;
 	}
 
+	/* Check to see if we have a trusted root issuer. */
 	for (i = 0; i < sk_X509_num(ctx->roots); i++) {
 		candidate = sk_X509_value(ctx->roots, i);
 		if (x509_verify_potential_parent(ctx, candidate, cert)) {
@@ -497,15 +523,7 @@ x509_verify_build_chains(struct x509_verify_ctx *ctx, X509 *cert,
 			    cert_md, 1, candidate, current_chain);
 		}
 	}
-	if (ctx->intermediates != NULL) {
-		for (i = 0; i < sk_X509_num(ctx->intermediates); i++) {
-			candidate = sk_X509_value(ctx->intermediates, i);
-			if (x509_verify_potential_parent(ctx, candidate, cert)) {
-				x509_verify_consider_candidate(ctx, cert,
-				    cert_md, 0, candidate, current_chain);
-			}
-		}
-	}
+	/* Check for legacy mode roots */
 	if (ctx->xsc != NULL) {
 		if ((ret = ctx->xsc->get_issuer(&candidate, ctx->xsc, cert)) < 0) {
 			x509_verify_cert_error(ctx, cert, depth,
@@ -521,6 +539,17 @@ x509_verify_build_chains(struct x509_verify_ctx *ctx, X509 *cert,
 		}
 	}
 
+	/* Check intermediates after checking roots */
+	if (ctx->intermediates != NULL) {
+		for (i = 0; i < sk_X509_num(ctx->intermediates); i++) {
+			candidate = sk_X509_value(ctx->intermediates, i);
+			if (x509_verify_potential_parent(ctx, candidate, cert)) {
+				x509_verify_consider_candidate(ctx, cert,
+				    cert_md, 0, candidate, current_chain);
+			}
+		}
+	}
+
 	if (ctx->chains_count > count) {
 		if (ctx->xsc != NULL) {
 			ctx->xsc->error = X509_V_OK;
@@ -528,7 +557,12 @@ x509_verify_build_chains(struct x509_verify_ctx *ctx, X509 *cert,
 			ctx->xsc->current_cert = cert;
 			(void) ctx->xsc->verify_cb(1, ctx->xsc);
 		}
-	} else if (ctx->error_depth == depth) {
+	} else if (ctx->error_depth == depth && !ctx->dump_chain) {
+		if (depth == 0 &&
+		    ctx->error == X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT_LOCALLY)
+			ctx->error = X509_V_ERR_UNABLE_TO_VERIFY_LEAF_SIGNATURE;
+		if (!x509_verify_ctx_set_xsc_chain(ctx, current_chain, 0, 0))
+			return;
 		(void) x509_verify_cert_error(ctx, cert, depth,
 		    ctx->error, 0);
 	}
@@ -604,8 +638,6 @@ x509_verify_asn1_time_to_tm(const ASN1_TIME *atime, struct tm *tm, int notafter)
 {
 	int type;
 
-	memset(tm, 0, sizeof(*tm));
-
 	type = ASN1_time_parse(atime->data, atime->length, tm, atime->type);
 	if (type == -1)
 		return 0;
@@ -689,11 +721,13 @@ x509_verify_validate_constraints(X509 *cert,
 		return 1;
 
 	if (cert->nc != NULL) {
-		if ((permitted = x509_constraints_names_new()) == NULL) {
+		if ((permitted = x509_constraints_names_new(
+		    X509_VERIFY_MAX_CHAIN_CONSTRAINTS)) == NULL) {
 			err = X509_V_ERR_OUT_OF_MEM;
 			goto err;
 		}
-		if ((excluded = x509_constraints_names_new()) == NULL) {
+		if ((excluded = x509_constraints_names_new(
+		    X509_VERIFY_MAX_CHAIN_CONSTRAINTS)) == NULL) {
 			err = X509_V_ERR_OUT_OF_MEM;
 			goto err;
 		}
@@ -722,6 +756,10 @@ x509_verify_cert_extensions(struct x509_verify_ctx *ctx, X509 *cert, int need_ca
 		CRYPTO_w_lock(CRYPTO_LOCK_X509);
 		x509v3_cache_extensions(cert);
 		CRYPTO_w_unlock(CRYPTO_LOCK_X509);
+		if (cert->ex_flags & EXFLAG_INVALID) {
+			ctx->error = X509_V_ERR_UNSPECIFIED;
+			return 0;
+		}
 	}
 
 	if (ctx->xsc != NULL)
@@ -1015,7 +1053,8 @@ x509_verify(struct x509_verify_ctx *ctx, X509 *leaf, char *name)
 		ctx->xsc->error = ctx->error;
 		if (ctx->chains_count > 0) {
 			/* Take the first chain we found. */
-			if (!x509_verify_ctx_set_xsc_chain(ctx, ctx->chains[0], 1))
+			if (!x509_verify_ctx_set_xsc_chain(ctx, ctx->chains[0],
+			    1, 1))
 				goto err;
 		}
 		return ctx->xsc->verify_cb(ctx->chains_count > 0, ctx->xsc);
