@@ -1,4 +1,4 @@
-/* $OpenBSD: d1_both.c,v 1.63 2020/12/05 19:34:57 tb Exp $ */
+/* $OpenBSD: d1_both.c,v 1.68 2021/02/27 14:20:50 jsing Exp $ */
 /*
  * DTLS implementation written by Nagendra Modadugu
  * (nagendra@cs.stanford.edu) for the OpenSSL project 2005.
@@ -201,12 +201,6 @@ dtls1_hm_fragment_free(hm_fragment *frag)
 	if (frag == NULL)
 		return;
 
-	if (frag->msg_header.is_ccs) {
-		EVP_CIPHER_CTX_free(
-		    frag->msg_header.saved_retransmit_state.enc_write_ctx);
-		EVP_MD_CTX_free(
-		    frag->msg_header.saved_retransmit_state.write_hash);
-	}
 	free(frag->fragment);
 	free(frag->reassembly);
 	free(frag);
@@ -218,7 +212,8 @@ dtls1_do_write(SSL *s, int type)
 {
 	int ret;
 	int curr_mtu;
-	unsigned int len, frag_off, mac_size, blocksize;
+	unsigned int len, frag_off;
+	size_t overhead;
 
 	/* AHA!  Figure out the MTU, and stick to the right size */
 	if (D1I(s)->mtu < dtls1_min_mtu() &&
@@ -246,21 +241,13 @@ dtls1_do_write(SSL *s, int type)
 		OPENSSL_assert(s->internal->init_num ==
 		    (int)D1I(s)->w_msg_hdr.msg_len + DTLS1_HM_HEADER_LENGTH);
 
-	if (s->internal->write_hash)
-		mac_size = EVP_MD_CTX_size(s->internal->write_hash);
-	else
-		mac_size = 0;
-
-	if (s->internal->enc_write_ctx &&
-	    (EVP_CIPHER_mode( s->internal->enc_write_ctx->cipher) & EVP_CIPH_CBC_MODE))
-		blocksize = 2 * EVP_CIPHER_block_size(s->internal->enc_write_ctx->cipher);
-	else
-		blocksize = 0;
+	if (!tls12_record_layer_write_overhead(s->internal->rl, &overhead))
+		return -1;
 
 	frag_off = 0;
 	while (s->internal->init_num) {
 		curr_mtu = D1I(s)->mtu - BIO_wpending(SSL_get_wbio(s)) -
-		    DTLS1_RT_HEADER_LENGTH - mac_size - blocksize;
+		    DTLS1_RT_HEADER_LENGTH - overhead;
 
 		if (curr_mtu <= DTLS1_HM_HEADER_LENGTH) {
 			/* grr.. we could get an error if MTU picked was wrong */
@@ -268,7 +255,7 @@ dtls1_do_write(SSL *s, int type)
 			if (ret <= 0)
 				return ret;
 			curr_mtu = D1I(s)->mtu - DTLS1_RT_HEADER_LENGTH -
-			    mac_size - blocksize;
+			    overhead;
 		}
 
 		if (s->internal->init_num > curr_mtu)
@@ -276,7 +263,6 @@ dtls1_do_write(SSL *s, int type)
 		else
 			len = s->internal->init_num;
 
-
 		/* XDTLS: this function is too long.  split out the CCS part */
 		if (type == SSL3_RT_HANDSHAKE) {
 			if (s->internal->init_off != 0) {
@@ -399,7 +385,7 @@ dtls1_get_message(SSL *s, int st1, int stn, int mt, long max, int *ok)
 		if ((mt >= 0) && (S3I(s)->tmp.message_type != mt)) {
 			al = SSL_AD_UNEXPECTED_MESSAGE;
 			SSLerror(s, SSL_R_UNEXPECTED_MESSAGE);
-			goto f_err;
+			goto fatal_err;
 		}
 		*ok = 1;
 		s->internal->init_msg = s->internal->init_buf->data + DTLS1_HM_HEADER_LENGTH;
@@ -441,7 +427,7 @@ again:
 	s->internal->init_msg = s->internal->init_buf->data + DTLS1_HM_HEADER_LENGTH;
 	return s->internal->init_num;
 
-f_err:
+ fatal_err:
 	ssl3_send_alert(s, SSL3_AL_FATAL, al);
 	*ok = 0;
 	return -1;
@@ -784,7 +770,7 @@ again:
 	    dtls1_get_message_header(wire, &msg_hdr) == 0) {
 		al = SSL_AD_UNEXPECTED_MESSAGE;
 		SSLerror(s, SSL_R_UNEXPECTED_MESSAGE);
-		goto f_err;
+		goto fatal_err;
 	}
 
 	/*
@@ -826,12 +812,12 @@ again:
 		{
 			al = SSL_AD_UNEXPECTED_MESSAGE;
 			SSLerror(s, SSL_R_UNEXPECTED_MESSAGE);
-			goto f_err;
+			goto fatal_err;
 		}
 	}
 
 	if ((al = dtls1_preprocess_fragment(s, &msg_hdr, max)))
-		goto f_err;
+		goto fatal_err;
 
 	/* XDTLS:  ressurect this when restart is in place */
 	S3I(s)->hs.state = stn;
@@ -857,7 +843,7 @@ again:
 	if (i != (int)frag_len) {
 		al = SSL3_AD_ILLEGAL_PARAMETER;
 		SSLerror(s, SSL3_AD_ILLEGAL_PARAMETER);
-		goto f_err;
+		goto fatal_err;
 	}
 
 	*ok = 1;
@@ -871,7 +857,7 @@ again:
 	s->internal->init_num = frag_len;
 	return frag_len;
 
-f_err:
+ fatal_err:
 	ssl3_send_alert(s, SSL3_AL_FATAL, al);
 	s->internal->init_num = 0;
 
@@ -985,8 +971,6 @@ dtls1_buffer_message(SSL *s, int is_ccs)
 	frag->msg_header.is_ccs = is_ccs;
 
 	/* save current state*/
-	frag->msg_header.saved_retransmit_state.enc_write_ctx = s->internal->enc_write_ctx;
-	frag->msg_header.saved_retransmit_state.write_hash = s->internal->write_hash;
 	frag->msg_header.saved_retransmit_state.session = s->session;
 	frag->msg_header.saved_retransmit_state.epoch = D1I(s)->w_epoch;
 
@@ -1017,7 +1001,6 @@ dtls1_retransmit_message(SSL *s, unsigned short seq, unsigned long frag_off,
 	unsigned long header_length;
 	unsigned char seq64be[8];
 	struct dtls1_retransmit_state saved_state;
-	unsigned char save_write_sequence[8];
 
 	/*
 	  OPENSSL_assert(s->internal->init_num == 0);
@@ -1063,18 +1046,9 @@ dtls1_retransmit_message(SSL *s, unsigned short seq, unsigned long frag_off,
 	/* restore state in which the message was originally sent */
 	s->session = frag->msg_header.saved_retransmit_state.session;
 	D1I(s)->w_epoch = frag->msg_header.saved_retransmit_state.epoch;
-	if (!tls12_record_layer_set_write_cipher_hash(s->internal->rl,
-	    frag->msg_header.saved_retransmit_state.enc_write_ctx,
-	    frag->msg_header.saved_retransmit_state.write_hash, 0))
-		return 0;
 
-	if (frag->msg_header.saved_retransmit_state.epoch ==
-	    saved_state.epoch - 1) {
-		memcpy(save_write_sequence, S3I(s)->write_sequence,
-		    sizeof(S3I(s)->write_sequence));
-		memcpy(S3I(s)->write_sequence, D1I(s)->last_write_sequence,
-		    sizeof(S3I(s)->write_sequence));
-	}
+	if (!tls12_record_layer_use_write_epoch(s->internal->rl, D1I(s)->w_epoch))
+		return 0;
 
 	ret = dtls1_do_write(s, frag->msg_header.is_ccs ?
 	    SSL3_RT_CHANGE_CIPHER_SPEC : SSL3_RT_HANDSHAKE);
@@ -1082,17 +1056,9 @@ dtls1_retransmit_message(SSL *s, unsigned short seq, unsigned long frag_off,
 	/* restore current state */
 	s->session = saved_state.session;
 	D1I(s)->w_epoch = saved_state.epoch;
-	if (!tls12_record_layer_set_write_cipher_hash(s->internal->rl,
-	    s->internal->enc_write_ctx, s->internal->write_hash, 0))
-		return 0;
 
-	if (frag->msg_header.saved_retransmit_state.epoch ==
-	    saved_state.epoch - 1) {
-		memcpy(D1I(s)->last_write_sequence, S3I(s)->write_sequence,
-		    sizeof(S3I(s)->write_sequence));
-		memcpy(S3I(s)->write_sequence, save_write_sequence,
-		    sizeof(S3I(s)->write_sequence));
-	}
+	if (!tls12_record_layer_use_write_epoch(s->internal->rl, D1I(s)->w_epoch))
+		return 0;
 
 	D1I(s)->retransmitting = 0;
 
@@ -1104,11 +1070,16 @@ dtls1_retransmit_message(SSL *s, unsigned short seq, unsigned long frag_off,
 void
 dtls1_clear_record_buffer(SSL *s)
 {
+	hm_fragment *frag;
 	pitem *item;
 
 	for(item = pqueue_pop(s->d1->sent_messages); item != NULL;
 	    item = pqueue_pop(s->d1->sent_messages)) {
-		dtls1_hm_fragment_free((hm_fragment *)item->data);
+		frag = item->data;
+		if (frag->msg_header.is_ccs)
+			tls12_record_layer_write_epoch_done(s->internal->rl,
+			    frag->msg_header.saved_retransmit_state.epoch);
+		dtls1_hm_fragment_free(frag);
 		pitem_free(item);
 	}
 }
